#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import math
import pygame
pygame.init()

from code.classes.Text import Text

class Radar( pygame.sprite.Sprite ):
	"""
	This is the radar class, the blueprint for the Lunar Rover 47 radar.
	"""

	def __init__( self, dimensions, bottom_right_position, tracking_area, color, transparency_color, keys, action_states, radar_on_text, radar_on_text_center_xy, radar_off_text, radar_off_text_center_xy, radar_info_text_font_size ):
		"""
		Parameters
			dimensions					( int, int ) or [ int, int ]
				Defines the dimensions of the image of the sprite.
			bottom_right_position 		[ int, int ] or ( int, int )
				Defines the position of the image of the sprite.
			tracking_area				int[ left, top, right, bottom ]
				Defines the area where objects can be detected by the radar.
				These will then be shown on the radar of the Lunar Rover 47.
			color						pygame.Color
				A color that is used to draw the dots onto the radar screen.
			transparency_color			pygame.Color
				Make the background of the Surface transparent.
			keys						dict [ pygame Key, string ]
				A dict to check if a key has been pressed.  The value is a string which can be used to perform some other action.
			action_states				dict [ string, bool ]
				A dict that consists of a string, which tells what to do if a key has been pressed and a boolean value, which decides whether to execute this action.
			radar_on_text				string	
				The text that will be displayed next to the radar to tell the user it is turned on.
			radar_on_text_center_xy		( int, int ) or [ int, int ]
				Where to blit the status to?
			radar_off_text				string
				The text that will be displayed next to the radar to tell the user it is turned off.
			radar_off_text_center_xy	( int, int ) or [ int, int ]
				Where to blit the status to?
			radar_info_text_font_size	int
				The size of the font used to create and display the radar on/off status message.
		"""
		
		pygame.sprite.Sprite.__init__( self )

		# I save the passed parameters, as I might need them again later on.
		self._dimensions = dimensions
		self._bottom_right_position = bottom_right_position
		self._tracking_area = pygame.Rect( tracking_area )
		self._color = color
		self._transparency_color = transparency_color
		self._keys = keys
		self._action_states = action_states
		self._radar_on_text = radar_on_text
		self._radar_on_text_center_xy = radar_on_text_center_xy
		self._radar_off_text = radar_off_text
		self._radar_off_text_center_xy = radar_off_text_center_xy
		self._radar_info_text_font_size = radar_info_text_font_size

		# Setting up the sprite.
		self.image = pygame.Surface( dimensions )
		self.image.set_colorkey( self._transparency_color )
		self.rect = self.image.get_rect( bottomright = bottom_right_position )

		# Store the tracked objects in a list.
		self._tracked_objects = []

		# A variable that turns the radar either off or on.
		self._radar_is_on = False
		self._r_key_already_pressed = False


		# I set up the simple sprites for the radar's on -and off info texts.
		self._radar_on_info = Text( self._radar_on_text, self._radar_on_text_center_xy, font_size=radar_info_text_font_size )
		self._radar_off_info = Text( self._radar_off_text, self._radar_off_text_center_xy, font_size=radar_info_text_font_size )

		# Used to display the current radar status message.
		self._radar_info_text = pygame.sprite.Sprite()

		# Show the user whether the radar is turned on or off by printing a message to the screen next to the radar.
		self.which_radar_info_to_display()
		
	def update( self, all_moon_objects, pressed_events ):
		"""
		Parameters
			all_moon_objects 	[ Object, Object, ... ]
				These are all the objects in the current level.  Some of these will be tracked, if they are within the tracking area.
			pressed_events		Python dict.
				A dictionary that tells which keys are currently held down.
		"""

		# Check if the radar should either be turned on or off.
		self.check_keys( pressed_events )
		self.check_action_states()

		# Avoid streaks.
		self.image.fill( self._transparency_color )

		# Filter objects that are within the area that can be tracked by the radar.
		self.get_moon_objects_to_track( all_moon_objects )

		# And draw those onto the radar screen.  Only draw them onto the radar screen if the radar is turned on.
		if self._radar_is_on:
			self.draw_onto_radar_screen()

		# Show the user whether the radar is turned on or off by printing a message to the screen next to the radar.
		self.which_radar_info_to_display()
		

	def get_moon_objects_to_track( self, all_moon_objects ):
		"""
		This function updates "self._tracked_objects" by filtering the objects of "all_moon_objects" that can be detected with the radar.
		The radar tracking dimensions are stored in "self._tracking_area".

		Parameters
			all_moon_objects [ Object, Object, ... ]
				These are all the objects of the current level that can be tracked.  Some of these actually will be tracked if they are within the detection range.
		"""

		# The list tracking the objects will be emptied every single frame.
		self._tracked_objects = []

		for obj in all_moon_objects:
			if self._tracking_area.collidepoint( obj.rect.center ):
				self._tracked_objects.append( obj )

	def draw_onto_radar_screen( self ):
		"""
		Draws the detected objects onto the radar screen.
		Since the radar is 160px x 120px in size, all coords of the objects need to be rescaled.

		This means, that I divide each coordinate by 4.  No 8.
		"""

		for obj in self._tracked_objects:
			# Scale coords.  See description above about why I do that.
			scaled_coords = [ ( 320 + obj.rect.centerx ) / 8, ( 240 + obj.rect.centery ) / 8 ]

 			# Represents the object as a dot on the radar.			
			pygame.draw.circle( self.image, self._color, scaled_coords, 1, 0 )


	def check_keys( self, pressed_keys ):
		"""
		This method checks the state of the keys and whether the radar should do something or not.

		Parameters
			pressed_keys	pygame.Event list.
				A list of key events that have happened in the current frame.
		"""

		for key_value_pair in self._keys.iteritems():
			if pressed_keys[key_value_pair[0]] and not self._r_key_already_pressed:
				self._action_states[key_value_pair[1]] = True
				self._r_key_already_pressed = True
			else:
				if not pressed_keys[key_value_pair[0]]:
					self._r_key_already_pressed = False


	def check_action_states( self ):
		"""
		Decides if specific actions, such as deactivating or activating the radar should be executed.
		"""

		if self._action_states["turn_radar_off_on"]:
			# Turn the radar either on or off.
			if self._radar_is_on == True:
				self._radar_is_on = False
			else:
				self._radar_is_on = True
				
			self._action_states["turn_radar_off_on"] = False


	def which_radar_info_to_display( self ):
		"""
		Based on whether the radar is turned on or off, another message is displayed on the window next to the radar.
		"""
		
		if self._radar_is_on:
			self._radar_info_text = self._radar_on_info
		else:
			self._radar_info_text = self._radar_off_info		
